#[macro_use]
extern crate structopt;

#[macro_use]
extern crate derive_builder;

mod controller;
mod model;
mod opts;

use controller::Userspace;
use model::WireguardController;
use opts::Opts;
use std::io;
use std::thread::sleep;
use std::time::{Duration, Instant};
use structopt::StructOpt;

fn main() -> io::Result<()> {
    let opts = Opts::from_args();
    let mut controller: Box<WireguardController> = Box::new(Userspace::new(opts.socket.clone()));
    let interval = Duration::from_millis(opts.poll);
    let timeout = Duration::from_secs(opts.timeout);
    println!(
        "Polling {} every {:?}",
        opts.socket.to_str().unwrap(),
        interval
    );
    loop {
        let now = Instant::now();
        let peers = controller.peers()?;
        println!("Connected peers:");
        for peer in peers {
            let peer = peer?;
            dbg!(peer.last_handshake);
            if peer.last_handshake.map(|h| h < timeout).unwrap_or(false) {
             println!("{:?}", peer.key);
            }
        }
        let pause = interval - now.elapsed();
        dbg!(pause);
        sleep(if pause > interval / 2 {
            pause
        } else {
            interval
        });
    }
    Ok(())
}
